﻿using UnityEngine;


/// <summary>
/// 弾丸
/// </summary>
public class Bullet : MonoBehaviour {
	#region MEMBER
	public GameObject explosion = null;		  // 爆発オブジェクト
	public bool isRotation = false;			  // 回転フラグ

	private Vector3 direct = Vector3.forward;   // 進行方向
	private float speed = 50f;                  // 移動速度（m/sec.）
	private SpriteRenderer spRender = null;     // 描画コンポーネント
	private bool isRender = false;              // 描画フラグ
	#endregion


	#region MAIN FUNCTION
	/// <summary>
	/// 初期化
	/// </summary>
	public void Initalize() {
		this.spRender = this.GetComponent<SpriteRenderer>();
	}

	/// <summary>
	/// 毎フレームの更新
	/// </summary>
	/// <param name="elapsedTime">経過時間</param>
	public bool Run(int index, float elapsedTime) {
		// 移動距離の計算
		Vector3 move = this.direct * (speed * elapsedTime);
		// 移動の反映
		this.transform.Translate(move, Space.World);
		// 描画プライオリティの更新
		this.spRender.sortingOrder = index;

		// 回転処理
		if (this.isRotation)
			this.transform.Rotate(0f, 0f, 1080f * elapsedTime);

		bool ret = this.isRender;
		this.isRender = false;  // 毎フレーム使い切り
		return ret;
	}

	/// <summary>
	/// 撮影範囲に入っているカメラ毎に
	/// Updateの前に呼び出されるMonoBehaviourのコールバック
	/// </summary>
	private void OnWillRenderObject() {
		this.isRender |= (Camera.current.cameraType == CameraType.Game);
	}

	/// <summary>
	/// 接触時のコールバック
	/// </summary>
	/// <param name="other">接触先</param>
	private void OnTriggerEnter(Collider other) {
		this.Vanish();
	}
	#endregion


	#region PUBLIC FUNCTION
	/// <summary>
	/// 外部（今回は自機）から向きと速度を設定
	/// </summary>
	/// <param name="position">座標</param>
	/// <param name="rotation">向き</param>
	/// <param name="speed">速度（m/sec.）</param>
	/// <param name="passedTime">超過時間（sec.）</param>
	/// <param name="sortOrder">描画プライオリティ</param>
	public void Shoot(Vector3 position, Quaternion rotation, float speed, float passedTime, int sortOrder) {
		this.direct = rotation * Vector3.up;
		this.speed = speed;
		// SpriteRendererはZ軸の負の方向が面の向きなのでX軸回転90度
		// 超過時間分の移動距離の計算
		position += this.direct * (speed * passedTime);
		// 弾丸の座標と回転値を反映
		this.transform.SetPositionAndRotation(position, rotation);
		// 描画プライオリティ設定
		this.spRender.sortingOrder = sortOrder;

		// 表示有効
		this.isRender = true;
		this.gameObject.SetActive(true);
	}

	/// <summary>
	/// 消滅
	/// </summary>
	public void Vanish(bool withFX = true) {
		// 消滅エフェクト
		if (withFX)
			Object.Instantiate(this.explosion, this.transform.position, Quaternion.identity);

		this.isRender = false;
		this.gameObject.SetActive(false);
	}
	#endregion
}
